@@ -110,7 +110,10 @@ class Net_IMAP extends Net_IMAPProtocol {
      */
     function login($user, $pass, $useauthenticate = true, $selectMailbox=true)
     {
+        //error_log(__METHOD__.':'.$user.','.$pass.','.$useauthenticate.','.$selectMailbox);
         if ( $useauthenticate ){
+            //$useauthenticate = 'LOGIN';
+            //error_log(__METHOD__.':'.'about to authenticate');
             //$useauthenticate is a string if the user hardcodes an AUTHMethod
             // (the user calls $imap->login("user","password","CRAM-MD5"); for example!
 
@@ -125,6 +128,7 @@ class Net_IMAP extends Net_IMAPProtocol {
                     $this->_serverAuthMethods=null;
                 }
                 if($this->_serverAuthMethods == null  || count($commonMethods) == 0 || $this->supportedAuthMethods == null ){
+					//error_log(__METHOD__.":The server does not have any auth method, so I try LOGIN");
                     // The server does not have any auth method, so I try LOGIN
                     if ( PEAR::isError( $ret = $this->cmdLogin( $user, $pass ) ) ) {
                         return $ret;
@@ -137,6 +141,7 @@ class Net_IMAP extends Net_IMAPProtocol {
                 return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
             }
         }else{
+            //error_log(__METHOD__.':'.'about to use plain login');
             //The user request "PLAIN"  auth, we use the login command
             if ( PEAR::isError( $ret = $this->cmdLogin( $user, $pass ) ) ) {
                 return $ret;
@@ -145,7 +150,7 @@ class Net_IMAP extends Net_IMAPProtocol {
                 return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
             }
         }
-
+        $this->cmdCapability();
         if($selectMailbox){
             //Select INBOX
             if ( PEAR::isError( $ret=$this->cmdSelect( $this->getCurrentMailbox() ) ) ) {
@@ -183,9 +188,13 @@ class Net_IMAP extends Net_IMAPProtocol {
         }
 
         if (PEAR::isError($ret=$this->cmdLogout())) {
+            error_log(__METHOD__.$ret->message);
             return $ret;
+        } else {
+	        //error_log(__METHOD__.print_r($ret,true));
         }
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
+            error_log(__METHOD__.$ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
 
@@ -256,9 +265,11 @@ class Net_IMAP extends Net_IMAPProtocol {
     function getRawHeaders($msg_id, $part_id = '', $uidFetch = false)
     {
         if($part_id != '') {
-          $command = "BODY[$part_id.HEADER]";
+          $command = "BODY.PEEK[$part_id.HEADER]";
+          $resp_command = 'BODY[' . $part_id . '.HEADER]';
         } else {
-          $command = "BODY[HEADER]";
+          $command = "BODY.PEEK[HEADER]";
+          $resp_command = 'BODY[HEADER]';
         }
         if($uidFetch == true) {
           $ret=$this->cmdUidFetch($msg_id, $command);
@@ -268,10 +279,16 @@ class Net_IMAP extends Net_IMAPProtocol {
         if (PEAR::isError($ret)) {
             return $ret;
         }
-        if(strtoupper( $ret["RESPONSE"]["CODE"]) != "OK" ){
+        if (strtoupper($ret["RESPONSE"]["CODE"]) != "OK" && count($ret['PARSED'])) error_log(__METHOD__.__LINE__.' ResposeCode not OK but found:'.count($ret['PARSED']).' parsed Responses for '.$msg_id.' PartId'.$part_id);
+        if(strtoupper( $ret["RESPONSE"]["CODE"]) != "OK" && !empty($ret["RESPONSE"]["CODE"])){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
-        $ret=$ret["PARSED"][0]["EXT"][$command]["CONTENT"];
+        foreach($ret["PARSED"] as $key => $value)
+        {
+            if (isset($ret["PARSED"][$key]["EXT"][$command]["CONTENT"])) {$found = $key; break;}
+            if (isset($ret["PARSED"][$key]["EXT"][$resp_command]["CONTENT"])) {$found = $key; break;}
+        }
+        $ret=($ret["PARSED"][$found]["EXT"][$resp_command]["CONTENT"]?$ret["PARSED"][$found]["EXT"][$resp_command]["CONTENT"]:$ret["PARSED"][$found]["EXT"][$command]["CONTENT"]);
         return $ret;
     }
 
@@ -373,13 +390,19 @@ class Net_IMAP extends Net_IMAPProtocol {
     {
        if( $msg_id != null){
             if(is_array($msg_id)){
+                //error_log(__METHOD__.__LINE__.' Request for Summary of '.count($msg_id).' IDs');
                 $message_set=$this->_getSearchListFromArray($msg_id);
             }else{
+                //error_log(__METHOD__.__LINE__.' Request for Summary of '.count(explode(',',$msg_id)).' IDs:'.$msg_id);
                 $message_set=$msg_id;
             }
         }else{
             $message_set="1:*";
         }
+        // mailbox needs to be selected immediately before a Fetch
+        $mailbox = $this->getCurrentMailbox();
+        if (!empty($mailbox)) $m = $this->cmdSelect($mailbox,false);
+        //error_log(__METHOD__.__LINE__.' Folder:'.array2string($mailbox).' Response:'.array2string($m));
         if($uidFetch) {
 		  #error_log("egw-pear::NET::IMAP:getSummary->fetch by UID ".$message_set);
           $ret=$this->cmdUidFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
@@ -387,39 +410,55 @@ class Net_IMAP extends Net_IMAPProtocol {
 		  #error_log("egw-pear::NET::IMAP:getSummary->fetch message ".$message_set);
           $ret=$this->cmdFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
         }
-		#error_log(print_r($ret['PARSED'][0],true));
+        //error_log(print_r($ret,true));
         #$ret=$this->cmdFetch($message_set,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY[1.MIME])");
-        if (PEAR::isError($ret) || strtoupper($ret["RESPONSE"]["CODE"]) != "OK") {
-			error_log("egw-pear::NET::IMAP:getSummary->error after Fetch for message(s):".$message_set." Trying to retrieve single messages.");
-			unset($ret);
-			# if there is an error, while retrieving the information for the whole list, try to retrieve the info one by one, to be more error tolerant
-			foreach (explode(',',$message_set) as $msgid) {
-				$retloop=$this->cmdUidFetch($msgid,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
-				if (PEAR::isError($retloop)|| strtoupper($retloop["RESPONSE"]["CODE"]) != "OK") {
-					# log the error, and create a dummy-message as placeholder, this may hold the possibility to read the message anyway
-					error_log("egw-pear::NET::IMAP:getSummary->error after Fetch for message with id:".$msgid);
-					error_log($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
-					$ret['PARSED'][]=array('COMMAND'=>"FETCH",'EXT'=>array('UID'=>$msgid,'ENVELOPE'=>array('SUBJECT'=>"[FELAMIMAIL:ERROR]can not parse this message(header).",)));
-				} else {
-					#error_log(print_r($retloop['PARSED'][0],true));
-					# renew the response for every message retrieved, since the returnvalue is structured that way
-					$ret['RESPONSE']=$retloop['RESPONSE'];
-					$ret['PARSED'][]=$retloop['PARSED'][0];
-				}
-				unset($retloop);
-			}
+        if ((PEAR::isError($ret) || (strtoupper($ret["RESPONSE"]["CODE"]) == "BAD" && stripos($ret["RESPONSE"]["STR_CODE"], "NO MAILBOX SELECTED")!==false) && $message_set=="1:*")) return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+        if (strtoupper($ret["RESPONSE"]["CODE"]) != "OK" && count($ret['PARSED'])) error_log(__METHOD__.__LINE__.' ResposeCode not OK but found:'.count($ret['PARSED']).' parsed Responses. Number of UIDs Requested:'.(is_array($msg_id)?count($msg_id):count(explode(',',$msg_id))).'. Trying to recover!');
+        $num_requested = (is_array($msg_id)?count($msg_id):count(explode(',',$msg_id)));
+        if (PEAR::isError($ret) || !isset($ret['PARSED']) ||
+            (strtoupper($ret["RESPONSE"]["CODE"]) != "OK" && !count($ret['PARSED'])) ||
+            (strtoupper($ret["RESPONSE"]["CODE"]) != "OK" && count($ret['PARSED']) < $num_requested)) {
+            //error_log(__METHOD__."::".__LINE__."->error after Fetch for message(s):".$message_set.' Result retrieved:->'.(PEAR::isError($ret)?$ret->message:count($ret['PARSED']))." Trying to retrieve (remaining) single messages.");
+            if (PEAR::isError($ret) || !isset($ret['PARSED'])) unset($ret);
+            else
+            {
+                //reduce message set, by the messages found
+                $mtR = explode(',',$message_set);
+				$mtR = array_flip($mtR);
+                for($i=0; $i<count($ret["PARSED"]) ; $i++){
+                    if (isset($ret["PARSED"][$i]['EXT']['UID']) && isset($mtR[$ret["PARSED"][$i]['EXT']['UID']])) unset($mtR[$ret["PARSED"][$i]['EXT']['UID']]);
+                }
+				if (!empty($mtR)) $message_set = $this->_getSearchListFromArray(array_keys($mtR));
+            }
+            # if there is an error, while retrieving the information for the whole list, try to retrieve the info one by one, to be more error tolerant
+            foreach (explode(',',$message_set) as $msgid) {
+                $retloop=$this->cmdUidFetch($msgid,"(RFC822.SIZE UID FLAGS ENVELOPE INTERNALDATE BODY.PEEK[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)])");
+                if (PEAR::isError($retloop)|| (strtoupper($retloop["RESPONSE"]["CODE"]) != "OK" && !count($ret['PARSED']))) {
+                    # log the error, and create a dummy-message as placeholder, this may hold the possibility to read the message anyway
+                    //error_log("egw-pear::NET::IMAP:getSummary->error after Fetch for message with id:".$msgid);
+                    error_log(__METHOD__.__LINE__."error after Fetch for message with id:".$msgid.' Error:'.$retloop["RESPONSE"]["CODE"] . ", " . $retloop["RESPONSE"]["STR_CODE"]);
+                    $ret['PARSED'][]=array('COMMAND'=>"FETCH",'EXT'=>array('UID'=>$msgid,'ENVELOPE'=>array('SUBJECT'=>"[FELAMIMAIL:ERROR]can not parse this message(header).",)));
+                } else {
+                    //error_log(print_r($retloop['PARSED'][0],true));
+                    //error_log(__METHOD__.__LINE__.' Fetched:'.$retloop["PARSED"][0]['EXT']['UID']);
+                    # renew the response for every message retrieved, since the returnvalue is structured that way
+                    $ret['RESPONSE']=$retloop['RESPONSE'];
+                    $ret['PARSED'][]=$retloop['PARSED'][0];
+                }
+                unset($retloop);
+            }
             #return $ret;
         }
-		# this seems to be obsolet, since errors while retrieving header informations are 'covered' above
-        #if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
-		#	error_log("egw-pear::NET::IMAP:getSummary->ResponseCode not OK");
-        #    return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
-        #}
-
-        #print "<hr>"; 
-		#error_log("egw-pear::NET::IMAP:getSummary->".print_r($ret["PARSED"],TRUE)); 
-		#print "<hr>";
+        // this seems to be obsolet, since errors while retrieving header informations are 'covered' above
+        if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK" && !empty($ret["RESPONSE"]["CODE"]))
+        {
+            error_log(__METHOD__."::".__LINE__."->ResponseCode not OK ->".$ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"].function_backtrace());
+            return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+        }
 
+        //print "<hr>";
+        //if (PEAR::isError($ret)) error_log("egw-pear::NET::IMAP:getSummary->".print_r($ret->message,TRUE));
+        //print "<hr>";
         if(isset( $ret["PARSED"] ) ){
             for($i=0; $i<count($ret["PARSED"]) ; $i++){
 				if ($ret["PARSED"][$i]['COMMAND'] != 'FETCH') continue;
@@ -432,6 +471,10 @@ class Net_IMAP extends Net_IMAPProtocol {
                 if(isset($ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'])) {
                     if(preg_match('/content-type: (.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'], $matches)) {
                       $a['MIMETYPE']=strtolower($matches[1]);
+                    }
+                    // some clients mess with the content-type and omit the space between keyword and mimetype
+                    if(!isset($a['MIMETYPE']) && preg_match('/content-type:(.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'], $matches)) {
+                      $a['MIMETYPE']=strtolower($matches[1]);
                     }
 					// fetch the priority [CONTENT] => X-Priority: 5\r\nContent-Type: multipart/alternative;\r\n\tboundary="b1_61838a67749ca51b425e42489adced98"\r\n\r\n\n
                     if(preg_match('/x-priority: ([0-9])/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'], $matches)) {
@@ -441,6 +484,10 @@ class Net_IMAP extends Net_IMAPProtocol {
                     // some versions of cyrus send "CONTENT-TYPE" and CONTENT-TYPE only
                     if (preg_match('/content-type: (.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE" "X-PRIORITY")]']['CONTENT'], $matches)) {
                         $a['MIMETYPE']=strtolower($matches[1]);
+                    }
+                    // some clients mess with the content-type and omit the space between keyword and mimetype
+                    if(!isset($a['MIMETYPE']) && preg_match('/content-type:(.*);/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS (CONTENT-TYPE X-PRIORITY)]']['CONTENT'], $matches)) {
+                      $a['MIMETYPE']=strtolower($matches[1]);
                     }
 					//  fetch the priority [CONTENT] => X-Priority: 5\r\nContent-Type: multipart/alternative;\r\n\tboundary="b1_61838a67749ca51b425e42489adced98"\r\n\r\n\n
                     if (preg_match('/x-priority: ([0-9])/iU', $ret["PARSED"][$i]['EXT']['BODY[HEADER.FIELDS ("CONTENT-TYPE" "X-PRIORITY")]']['CONTENT'], $matches)) {
@@ -463,17 +510,19 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @param   $msg_id Message number
      * @param   boolean $uidFetch  msg_id contains UID's instead of Message Sequence Number if set to true
+     * @param   boolean $preserveSeen  leaves the Seen Flag untouched if set to true (default is false)
      *
      * @return  mixed   Either message body or false on error
      *
      * @access  public
      */
-    function getBody($msg_id, $uidFetch = false)
+    function getBody($msg_id, $uidFetch = false, $preserveSeen = false)
     {
+        $peek = ($preserveSeen) ? ".PEEK" : "";
         if($uidFetch) {
-          $ret=$this->cmdUidFetch($msg_id,"BODY[TEXT]");
+          $ret=$this->cmdUidFetch($msg_id,"BODY".$peek."[TEXT]");
         } else {
-          $ret=$this->cmdFetch($msg_id,"BODY[TEXT]");
+          $ret=$this->cmdFetch($msg_id,"BODY".$peek."[TEXT]");
         }
         if (PEAR::isError($ret)) {
             return $ret;
@@ -481,7 +530,12 @@ class Net_IMAP extends Net_IMAPProtocol {
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
-        $ret=$ret["PARSED"][0]["EXT"]["BODY[TEXT]"]["CONTENT"];
+        $found = 0;
+        foreach($ret["PARSED"] as $key => $value)
+        {
+            if (isset($ret["PARSED"][$key]["EXT"]["BODY[TEXT]"]["CONTENT"])) {$found = $key; break;}
+        }
+        $ret=$ret["PARSED"][$found]["EXT"]["BODY[TEXT]"]["CONTENT"];
         //$ret=$resp["PARSED"][0]["EXT"]["RFC822"]["CONTENT"];
         return $ret;
     }
@@ -493,17 +547,19 @@ class Net_IMAP extends Net_IMAPProtocol {
      * @param   $msg_id Message number
      * @param   string  $partId Message number
      * @param   boolean $uidFetch  msg_id contains UID's instead of Message Sequence Number if set to true
+     * @param   boolean $preserveSeen  leaves the Seen Flag untouched if set to true (default is false)
      *
      * @return  mixed   Either message body or false on error
      *
      * @access  public
      */
-    function getBodyPart($msg_id, $partId, $uidFetch = false)
+    function getBodyPart($msg_id, $partId, $uidFetch = false, $preserveSeen = false)
     {
+        $peek = ($preserveSeen) ? ".PEEK" :  "" ;
         if($uidFetch) {
-          $ret=$this->cmdUidFetch($msg_id,"BODY[$partId]");
+          $ret=$this->cmdUidFetch($msg_id,"BODY".$peek."[$partId]");
         } else {
-          $ret=$this->cmdFetch($msg_id,"BODY[$partId]");
+          $ret=$this->cmdFetch($msg_id,"BODY".$peek."[$partId]");
         }
         if (PEAR::isError($ret)) {
             return $ret;
@@ -511,7 +567,12 @@ class Net_IMAP extends Net_IMAPProtocol {
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
-        $ret=$ret["PARSED"][0]["EXT"]["BODY[$partId]"]["CONTENT"];
+        $found = 0;
+        foreach($ret["PARSED"] as $key => $value)
+        {
+            if (isset($ret["PARSED"][$key]["EXT"]["BODY[$partId]"]["CONTENT"])) {$found = $key; break;}
+        }
+        $ret=$ret["PARSED"][$found]["EXT"]["BODY[$partId]"]["CONTENT"];
         //$ret=$resp["PARSED"][0]["EXT"]["RFC822"]["CONTENT"];
         return $ret;
     }
@@ -538,26 +599,34 @@ class Net_IMAP extends Net_IMAPProtocol {
         } else {
           $ret=$this->cmdFetch($msg_id,"BODYSTRUCTURE");
         }
-		#_debug_array($ret);
+        #_debug_array($ret);
         if (PEAR::isError($ret)) {
             return $ret;
         }
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
-            return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+            if ($ret["RESPONSE"]["CODE"]) return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+        }
+        $found = 0;
+        foreach($ret["PARSED"] as $key => $value)
+        {
+            if (isset($ret["PARSED"][$key]["EXT"]["BODYSTRUCTURE"][0])) {$found = $key; break;}
+        }
+        $ret2=$ret["PARSED"][$found]["EXT"]["BODYSTRUCTURE"][0];
+        // sometimes we get an [COMMAND] => OK with $ret["PARSED"][0] and no $ret["PARSED"][0]["EXT"]["BODYSTRUCTURE"]
+        // should not happen anymore
+        if (is_array($ret) && empty($ret2) && isset($ret["PARSED"])) {
+            foreach($ret["PARSED"] as $substruct) {
+                if ($substruct["COMMAND"] == "FETCH") {
+                    $ret2=$substruct["EXT"]["BODYSTRUCTURE"][0];
+                    break;
+                }
+            }
         }
-        $ret2=$ret["PARSED"][0]["EXT"]["BODYSTRUCTURE"][0];
-		// sometimes we get an [COMMAND] => OK with $ret["PARSED"][0] and no $ret["PARSED"][0]["EXT"]["BODYSTRUCTURE"]
-		if (is_array($ret) && empty($ret2)) {
-			foreach($ret["PARSED"] as $substruct) {
-				if ($substruct["COMMAND"] == "FETCH") {
-					$ret2=$substruct["EXT"]["BODYSTRUCTURE"][0];
-					break;
-				}
-			}
-		}
         $structure = array();
 
         $mimeParts = array();
+
+        if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK" && empty($ret2)) return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"].'/'.'No BODYSTRUCTURE found!');
         $this->_parseStructureArray($ret2, $mimeParts);
         #_debug_array($ret);
         return array_shift($mimeParts);
@@ -575,7 +644,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @access  private
      */
-    function _parseStructureArray($_structure, &$_mimeParts, $_partID = '') 
+    function _parseStructureArray($_structure, &$_mimeParts, $_partID = '')
     {
         // something went wrong
         if(!is_array($_structure)) {
@@ -603,9 +672,9 @@ class Net_IMAP extends Net_IMAPProtocol {
               break;
           }
         }
-        
+
     }
-    
+
 
 
     /**
@@ -620,7 +689,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @access  private
      */
-    function _parseStructureMultipartArray($_structure, &$_mimeParts, $_partID, $_parentIsMessage = false) 
+    function _parseStructureMultipartArray($_structure, &$_mimeParts, $_partID, $_parentIsMessage = false)
     {
         #print "Net_IMAP::_parseStructureMultipartArray _partID: $_partID<br>";
         // a multipart/mixed, multipart/report or multipart/alternative or multipart/related get's no own partid, if the parent is message/rfc822
@@ -635,7 +704,7 @@ class Net_IMAP extends Net_IMAPProtocol {
             $_partID = substr($_partID, 0, strrpos($_partID, '.'));
           }
         }
-        
+
         $subPartID = 1;
         $partID = ($_partID == '') ? '' : $_partID.'.';
         $subMimeParts = array();
@@ -648,22 +717,22 @@ class Net_IMAP extends Net_IMAPProtocol {
               switch(strtoupper($structurePart[0])) {
                  case 'IMAGE':
                   $this->_parseStructureImageArray($structurePart, $subMimeParts, $partID.$subPartID);
-                  
+
                   break;
 
                  case 'MESSAGE':
                   $this->_parseStructureMessageArray($structurePart, $subMimeParts, $partID.$subPartID);
-                  
+
                   break;
 
                 case 'TEXT':
                   $this->_parseStructureTextArray($structurePart, $subMimeParts, $partID.$subPartID);
-              
+
                   break;
 
                 default:
                   $this->_parseStructureApplicationArray($structurePart, $subMimeParts, $partID.$subPartID);
-              
+
                   break;
 
               }
@@ -673,9 +742,9 @@ class Net_IMAP extends Net_IMAPProtocol {
             $part = new stdClass;
             $part->type = 'MULTIPART';
             $part->subType = strtoupper($structurePart);
-          
+
             $part->subParts = $subMimeParts;
-          
+
             if($_partID == '') {
               $part->partID = 0;
               $_mimeParts = array(0 => $part);
@@ -683,7 +752,7 @@ class Net_IMAP extends Net_IMAPProtocol {
               $part->partID = $_partID;
               $_mimeParts[$_partID] = $part;
             }
-            
+
             return;
           }
         }
@@ -700,18 +769,31 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @access  private
      */
-    function _parseStructureImageArray($_structure, &$_mimeParts, $_partID) 
+    function _parseStructureImageArray($_structure, &$_mimeParts, $_partID)
     {
         #print "Net_IMAP::_parseStructureImageArray _partID: $_partID<br>";
         $part = $this->_parseStructureCommonFields($_structure);
         $part->cid = $_structure[3];
         $part->partID = $_partID;
-        
+        // there may be a part 8 for images too, ...
+        if(is_array($_structure[8])) {
+          if(isset($_structure[8][0]) && $_structure[8][0] != 'NIL') {
+            $part->disposition = strtoupper($_structure[8][0]);
+          }
+          if(is_array($_structure[8][1])) {
+            foreach($_structure[8][1] as $key => $value) {
+              if($key%2 == 0) {
+                $part->dparameters[trim(strtoupper($_structure[8][1][$key]))] = $_structure[8][1][$key+1];
+              }
+            }
+          }
+        }
+
         $_mimeParts[$_partID] = $part;
     }
 
 
-    
+
     /**
      * Parse structure application array
      *
@@ -723,7 +805,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @access  private
      */
-    function _parseStructureApplicationArray($_structure, &$_mimeParts, $_partID) 
+    function _parseStructureApplicationArray($_structure, &$_mimeParts, $_partID)
     {
         #print "Net_IMAP::_parseStructureApplicationArray _partID: $_partID<br>";
         $part = $this->_parseStructureCommonFields($_structure);
@@ -734,13 +816,13 @@ class Net_IMAP extends Net_IMAPProtocol {
           if(is_array($_structure[8][1])) {
             foreach($_structure[8][1] as $key => $value) {
               if($key%2 == 0) {
-                $part->dparameters[strtoupper($_structure[8][1][$key])] = $_structure[8][1][$key+1];
+                $part->dparameters[trim(strtoupper($_structure[8][1][$key]))] = $_structure[8][1][$key+1];
               }
             }
           }
         }
         $part->partID = $_partID;
-        
+
         $_mimeParts[$_partID] = $part;
     }
 
@@ -761,13 +843,13 @@ class Net_IMAP extends Net_IMAPProtocol {
     {
         #print "Net_IMAP::_parseStructureMessageArray _partID: $_partID<br>";
         $part = $this->_parseStructureCommonFields($_structure);
-        
+
         if(is_array($_structure[8][0])) {
           $this->_parseStructureMultipartArray($_structure[8], $subMimeParts, $_partID.'.1', true);
         } else {
           $this->_parseStructureArray($_structure[8], $subMimeParts, $_partID);
         }
-        
+
         if(is_array($subMimeParts)) {
           $part->subParts = $subMimeParts;
         }
@@ -775,7 +857,7 @@ class Net_IMAP extends Net_IMAPProtocol {
 
         $_mimeParts[$_partID] = $part;
     }
-    
+
 
 
     /**
@@ -789,14 +871,14 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @access  private
      */
-    function _parseStructureTextArray($_structure, &$_mimeParts, $_partID) 
+    function _parseStructureTextArray($_structure, &$_mimeParts, $_partID)
     {
         #print "Net_IMAP::_parseStructureTextArray _partID: $_partID<br>";
         $part = $this->_parseStructureCommonFields($_structure);
         $part->lines = $_structure[7];
-        
+
         // what is the difference between $_structure[8] and $_structure[9]????
-        
+
         if(is_array($_structure[8])) {
           if(isset($_structure[8][0]) && $_structure[8][0] != 'NIL') {
             $part->disposition = strtoupper($_structure[8][0]);
@@ -804,7 +886,7 @@ class Net_IMAP extends Net_IMAPProtocol {
           if(is_array($_structure[8][1])) {
             foreach($_structure[8][1] as $key => $value) {
               if($key%2 == 0) {
-                $part->dparameters[strtoupper($_structure[8][1][$key])] = $_structure[8][1][$key+1];
+                $part->dparameters[trim(strtoupper($_structure[8][1][$key]))] = $_structure[8][1][$key+1];
               }
             }
           }
@@ -817,14 +899,14 @@ class Net_IMAP extends Net_IMAPProtocol {
           if(is_array($_structure[9][1])) {
             foreach($_structure[9][1] as $key => $value) {
               if($key%2 == 0) {
-                $part->dparameters[strtoupper($_structure[9][1][$key])] = $_structure[9][1][$key+1];
+                $part->dparameters[trim(strtoupper($_structure[9][1][$key]))] = $_structure[9][1][$key+1];
               }
             }
           }
         }
-        
+
         $part->partID = $_partID;
-        
+
         $_mimeParts[$_partID] = $part;
     }
 
@@ -839,7 +921,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @access  private
      */
-    function _parseStructureCommonFields(&$_structure) 
+    function _parseStructureCommonFields(&$_structure)
     {
 		#error_log(__METHOD__.print_r($_structure,true)." ".function_backtrace());
         #print "Net_IMAP::_parseStructureTextArray _partID: $_partID<br>";
@@ -850,21 +932,21 @@ class Net_IMAP extends Net_IMAPProtocol {
 		if(is_array($_structure[1])) {
 			foreach($_structure[1] as $key => $value) {
 				if($key%2 == 0) {
-					$part->parameters[strtoupper($_structure[1][$key])] = $_structure[1][$key+1];
+					$part->parameters[trim(strtoupper($_structure[1][$key]))] = $_structure[1][$key+1];
 				}
 			}
 		}
         if(is_array($_structure[2])) {
           foreach($_structure[2] as $key => $value) {
             if($key%2 == 0) {
-              $part->parameters[strtoupper($_structure[2][$key])] = $_structure[2][$key+1];
+              $part->parameters[strtoupper(trim($_structure[2][$key]))] = $_structure[2][$key+1];
             }
           }
         }
         $part->filename = $_structure[4];
         $part->encoding = strtoupper($_structure[5]);
         $part->bytes = $_structure[6];
-        
+
         return $part;
     }
 
@@ -1330,7 +1412,7 @@ class Net_IMAP extends Net_IMAPProtocol {
                 );
             }
         }
-        
+
         return $nameSpaces;
     }
 
@@ -1375,6 +1457,44 @@ class Net_IMAP extends Net_IMAPProtocol {
     }
 
 
+    /**
+     * Gets the SPECIAL-USE Folders
+     *
+     * $param   string  the mailbox to get the SPECIAL-USE Folders from
+     *
+     * @return  string  the hierarchy delimiter
+     *
+     * @access  public
+     * @since   1.0
+     */
+    function getSpecialUseFolders( $mailbox = '' )
+    {
+		$returnAttributes=true;
+        if( PEAR::isError( $ret = $this->cmdListSpecialUse( $mailbox , '*' )  ) ){
+            return $ret;
+        }
+		//error_log(__METHOD__.__LINE__.array2string($ret));
+        if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
+            return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
+        }
+        $ret_aux=array();
+        if( isset($ret["PARSED"]) ){
+            foreach( $ret["PARSED"] as $mbox ){
+                //If the folder has the \NoSelect atribute we don't put in the list
+                // it solves a bug in wu-imap that crash the IMAP server if we select that mailbox
+                if( isset($mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"]) ){
+                    if( $returnAttributes){
+                        $ret_aux[]=array(   'MAILBOX' => $mbox["EXT"]["LIST"]["MAILBOX_NAME"],
+                                            'ATTRIBUTES' => $mbox["EXT"]["LIST"]["NAME_ATTRIBUTES"] ,
+                                            //'HIERACHY_DELIMITER' => $mbox["EXT"]["LIST"]["HIERACHY_DELIMITER"]
+										) ;
+                    }
+                }
+            }
+        }
+        return $ret_aux;
+    }
+
 
     /**
      * Returns an array containing the names of the selected mailboxes
@@ -1524,9 +1644,9 @@ class Net_IMAP extends Net_IMAPProtocol {
         }
         return true;
     }
-    
-    
-    
+
+
+
     /**
      * Renames the mailbox $mailbox
      *
@@ -1665,15 +1785,14 @@ class Net_IMAP extends Net_IMAPProtocol {
             foreach( $ret["PARSED"] as $mbox ){
                 if( isset($mbox["EXT"]["LSUB"]["MAILBOX_NAME"]) ){
                     if( $returnAttributes){
-                            $ret_aux[]=array(
-                                        'MAILBOX' => $mbox["EXT"]["LSUB"]["MAILBOX_NAME"],
-                                        'ATTRIBUTES' => $mbox["EXT"]["LSUB"]["NAME_ATTRIBUTES"],
-                                        'HIERACHY_DELIMITER' =>  $mbox["EXT"]["LSUB"]["HIERACHY_DELIMITER"]
-                                        ) ;
-                        }else{
-                            $ret_aux[]=$mbox["EXT"]["LSUB"]["MAILBOX_NAME"];
-
-                        }
+                        $ret_aux[]=array(
+                                'MAILBOX' => $mbox["EXT"]["LSUB"]["MAILBOX_NAME"],
+                                'ATTRIBUTES' => $mbox["EXT"]["LSUB"]["NAME_ATTRIBUTES"],
+                                'HIERACHY_DELIMITER' =>  $mbox["EXT"]["LSUB"]["HIERACHY_DELIMITER"]
+                        ) ;
+                    } else {
+                        $ret_aux[]=$mbox["EXT"]["LSUB"]["MAILBOX_NAME"];
+                    }
                 }
             }
         }
@@ -1720,6 +1839,7 @@ class Net_IMAP extends Net_IMAPProtocol {
 		if (PEAR::isError($ret)) {
             return $ret;
         }
+
         if(strtoupper($ret["RESPONSE"]["CODE"]) != "OK"){
             return new PEAR_Error($ret["RESPONSE"]["CODE"] . ", " . $ret["RESPONSE"]["STR_CODE"]);
         }
@@ -1741,7 +1861,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      *
      * @param   mixed   $msg_id     the message list or string "all" for all
      * @param   mixed   $flags      flags to set (space separated String or array)
-     * @param   string  $mod        "set" to set flags (default) 
+     * @param   string  $mod        "set" to set flags (default)
      *                              "add" to add flags
      *                              "remove" to remove flags
      * @param   boolean $uidStore   msg_id contains UID's instead of Message Sequence Number if set to true
@@ -1764,14 +1884,14 @@ class Net_IMAP extends Net_IMAPProtocol {
                 $message_set = $msg_id;
             }
         }
-        
+
         $flaglist = '';
         if (is_array($flags)) {
             $flaglist = implode(' ', $flags);
         } else {
             $flaglist = $flags;
         }
-        
+
         switch ($mod) {
             case 'set':
                 $dataitem = 'FLAGS';
@@ -1787,7 +1907,7 @@ class Net_IMAP extends Net_IMAPProtocol {
                 return new PEAR_Error('wrong input $mod');
                 break;
         }
-        #error_log("egw-pear::Net::setFlags for Message: ".print_r($message_set,true)."->".$flaglist); 
+        #error_log("egw-pear::Net::setFlags for Message: ".print_r($message_set,true)."->".$flaglist);
         if($uidStore == true) {
           $ret=$this->cmdUidStore($message_set, $dataitem, $flaglist);
         } else {
@@ -1814,7 +1934,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      * @return  mixed   true on success/PearError on failure
      *
      * @since   1.1
-     * @access  public 
+     * @access  public
      */
     function addFlags($msg_id, $flags)
     {
@@ -2130,7 +2250,7 @@ class Net_IMAP extends Net_IMAPProtocol {
     **                                                               **
     ******************************************************************/
 
-    
+
     /**
      * expunge function. Sends the EXPUNGE command
      *
@@ -2194,7 +2314,7 @@ class Net_IMAP extends Net_IMAPProtocol {
      * @access  public
      * @since   1.1
      */
-    function sort($sort_list, $charset='US-ASCII', $search_list = '', $uidSort = false)
+    function sort($sort_list, $charset='US-ASCII', $search_list = 'ALL', $uidSort = false)
     {
         $sort_command = sprintf("(%s) %s %s", $sort_list, strtoupper($charset), $search_list);
 
@@ -2332,8 +2452,8 @@ class Net_IMAP extends Net_IMAPProtocol {
         }
         return array('USED'=>'NOT SET', 'QMAX'=>'NOT SET');
     }
-     
-    
+
+
 
     /**
      * sets STORAGE quota
@@ -2693,7 +2813,7 @@ class Net_IMAP extends Net_IMAPProtocol {
 
     *********************************************************/
 
-    
+
     /**
      * same as getMailboxSize()
      * Net_POP3 Compatibility function
@@ -2763,7 +2883,7 @@ class Net_IMAP extends Net_IMAPProtocol {
     }
 
 
-    
+
     /**
      * same as deleteMessages($msg_id)
      * Net_POP3 Compatibility function
